%% DC Motor Model
% Note: This file plots step responses of an ODE model for the
% Precision MicroDrives DC Motor model 112-002.  

%% Model Parameters

% Nominal ODE Coefficients
a0Nom = 0.94;                      % ODE coefficient, 1/sec
b0Nom = 766.8;                     % ODE coefficient, rad/(sec^2 V)

%% Nominal Model and Unit Step Response
% Input: Motor voltage u, V
% Output: Rotational Speed y, rad/sec

% Nominal Model
a0 = a0Nom;
b0 = b0Nom;
G = tf(b0,[1 a0]);    

% Model Info:  Pole, Time Constant, DC Gain
p = pole(G)
tau = 1/abs(p)
Gdc = dcgain(G)

% Step responses with u(t)=1V and u(t)=2V
% Note: If the response due to u(t)=1V is y1(t) then the response due
% to u(t)=2V is y2(t)=2*y1(t) by the linearity of the model.
Tsim=linspace(0,5*tau,1e2);
[y1,t]=step(G,Tsim);
y2 = 2*y1;

figure(1); clf;
plot(t,y1,'b',t,y2,'r-.');
xlabel('Time (sec)');
ylabel('Speed, y (rad/sec)');
legend('u(t)=1V','u(t)=2V','Location','Best')
grid on;
axis([0 Tsim(end) 0 1800]);
set(gca, 'LooseInset', get(gca, 'TightInset'));

if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    garyfyFigure
    myprint(1,'DCModel_Nom')
end

%% Simulations with Parameter Variations
% Simulate step responses with coefficient values selected within
% the range of uncertainty. Also overlay the nominal motor response.

% Plot step responses
figure(2); clf;
for i=1:5
    % Simulations 1-4 correspond to +/-10% uncertainty in the
    % ODE coefficients. Simulation 5 gives the nominal response.
    if i==1
        a0 = 1.1*a0Nom;
        b0 = 1.1*b0Nom;
    elseif i==2
        a0 = 1.1*a0Nom;
        b0 = 0.9*b0Nom;
    elseif i==3
        a0 = 0.9*a0Nom;
        b0 = 1.1*b0Nom;
    elseif i==4
        a0 = 0.9*a0Nom;
        b0 = 0.9*b0Nom;
    else
        % Use nominal values for last simulation
        a0 = a0Nom;
        b0 = b0Nom;
    end
    Gi = tf(b0,[1 a0]);
    
    % Plot step response
    [yi,ti] = step(Gi,Tsim);
    if i<5
        ph(i)=plot(ti,yi,'r-.');    
    else
        ph(i)=plot(ti,yi,'b');    
    end
    hold on;
end

% Add labels and legends
xlabel('Time (sec)');
ylabel('Speed, y (rad/sec)');
grid on;
xlim([0 Tsim(end)]);
legend([ph(5); ph(4)],'Nominal','+/-10% Variation','Location','Best');
set(gca, 'LooseInset', get(gca, 'TightInset'));

if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    garyfyFigure
    set(ph(1:4),'LineWidth',1.5);
    myprint(2,'DCModel_PV')
end

%% Simulations with fast dynamics
% Simulate step responses with nominal (first-order) response and
% second order model that includes fast electrical dynamics.

% Second-order model
rpm2radpsec = 2*pi/60;          % Conversion from rpm to rad/sec
urated = 2.4;                   % Rated Voltage, V
yNL = 17000*rpm2radpsec;        % No load speed at rated voltage, rad/sec
iNL = 270e-3;                   % No load current, Amps
R = 0.8;                        % Motor resistance, Ohms
L = 150e-6;                     % Motor inductance, H
J = 2e-6;                       % Inertia, kg*m^2
b = 0;                          % Damping friction const., N*m/(rad/sec)
Ke = (urated - R*iNL)/yNL;      % Back EMF constant, V/(rad/sec);
Kt = Ke;                        % Torque constant, N*m/Amp

G2 = tf(Kt,[J*L (J*R+b*L) (Kt*Ke+b*R)]); 

% Plot step responses
[y1,t1]=step(G,Tsim);
[y2,t2]=step(G2,Tsim);

figure(3); clf;
plot(t1,y1,'b'); 
hold on;
ph = plot(t2,y2,'r:'); 
hold off
xlabel('Time (sec)');
ylabel('Speed, y (rad/sec)');
legend('Nominal (1st Order)','Second-Order','Location','Best');
grid on;
xlim([0 Tsim(end)]);
set(gca, 'LooseInset', get(gca, 'TightInset'));

if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    garyfyFigure
    %set(ph,'LineWidth',1.5);
    myprint(3,'DCModel_HF')
end
