%% Rocket Attitude: Closed-Loop Proportional Derivative Control

%% Model Parameters

% Nominal ODE Coefficients
a1Nom = 0;                      % ODE coefficient, 1/sec
a0Nom = -0.12;                  % ODE coefficient, 1/sec^2
b0Nom = 6.32;                   % ODE coefficient, 1/sec^2

%% Nominal Model and Unit Step Response
% Input: Thrust angle, rad
% Output: Rocket heading angle, rad

% Nominal Model
a1 = a1Nom;
a0 = a0Nom;
b0 = b0Nom;
G = tf(b0,[1 a1 a0]);    

% Model Info:  Pole, Time Constant, DC Gain
p = pole(G)
tau = 1./abs(p)
Gdc = dcgain(G)

% Step response with u(t)=0.05 rad
% Note: If the response due to u(t)=1rad is y1(t) then the response due
% to u(t)=0.05rad is y(t)=0.05*y1(t) by the linearity of the model.
Tsim=linspace(0,5,1e2);
[y1,t]=step(G,Tsim);
y = 0.05*y1;

figure(1); clf;
plot(t,y,'b');
xlabel('Time (sec)');
ylabel('Attitude, y (rad)');
grid on;
%axis([0 Tsim(end) 0 1800]);
set(gca, 'LooseInset', get(gca, 'TightInset'));

if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    garyfyFigure
    myprint(1,'RocketModel_Nom')
end

%% Design PD Controller
% Use second order response characteristics to design the PD controller.

% Desired settling time and overshoot
ts = 2;         % Desired settling time, sec
Mp = 0.046;     % Desired peak overshoot, unitless

% Solve for damping ratio and natural frequency required to achieve
% the desired response characteristics. This uses the relations 
%   Mp = exp( -pi*zeta / sqrt(1-zeta^2)  )
%   ts = 3/(zeta*wn)
% Note: This neglects the effect of the zero at -Kp/Kd.
zeta = sqrt( log(Mp)^2/(pi^2+log(Mp)^2));   % Damping Ratio, unitless
wn = 3/zeta/ts;     % Natural frequency, rad/sec

% Solve for PD gains required to place poles with (wn,zeta).
Kp = (wn^2-a0)/b0;
Kd = (2*zeta*wn-a1)/b0;

%% Simulations on Nominal Model
% The inputs/outputs of the PD controller are:
%  Inputs:
%      1) Desired heading angle r, rad
%      2) Actual heading angle y, rad
%  Output: Motor voltage u, V
% The PD controller computes the error e=r-y and then selects the
% rocket thrust angle as u = Kp*e + Kd*d(e)/dt.


% Simulation and plot settings
ulim = inf;     % Limit on max motor voltage for simulation, V
d = -0.01;        % Input disturbance, V
Tf = 10;         % Final simulation time, sec
co = get(groot,'defaultAxesColorOrder');

% Simulate with PI controller
figure(2); clf;
figure(3); clf;
figure(4); clf;
figure(5); clf;

% Set plant numerator & denominator coefficients
Gnum = b0;       % Numerator coefficients of G
Gden = [1 a1 a0];   % Denominator coefficients of G

% Simulate Simulink model
r = 0.1;   % Desired heading angle, rad
sim('RocketPDControl')


%%
% Plot input and contributions of P and D terms.
% Include inset figure to show zoom of input signals during transient.
figure(2);
plot(t,u,'b',t,up,'r:',t,ud,'g-.');
legend('PD Input (Total)','P-term','D-term','Location','SouthEast')
xlabel('Time (sec)');
ylabel('Input u (rad)');
grid on;
axis([0 Tf -0.15 .3]);
set(gca, 'LooseInset', get(gca, 'TightInset'));

axes('position',[0.4 0.62 0.5 0.33])
plot(t,u,'b',t,up,'r:',t,ud,'g-.');
xlabel('Time (sec)');
ylabel('Input u (rad)');
grid on;
axis([0 0.1 -0.2 5]);
set(gca, 'LooseInset', get(gca, 'TightInset'));


%%
% Plot rocket heading angle
figure(3);
plot(t,y,'b',[0 Tf],[r r],'k:');
xlabel('Time (sec)');
ylabel('Heading Angle, y (rad)');
grid on;
set(gca, 'LooseInset', get(gca, 'TightInset'));

% Plot error
figure(4);
plot(t,e,'b');
xlabel('Time (sec)');
ylabel('Error, e (rad)');
grid on;
set(gca, 'LooseInset', get(gca, 'TightInset'));

% Plot locations of closed-loop poles and zeros
figure(5);
p = roots([1 a1+b0*Kd a0+b0*Kp]);
z = -Kp/Kd;
plot(real(p),imag(p),'bx',real(z),imag(z),'bo','MarkerSize',12);
xlabel('Real');
ylabel('Imag');
legend('Poles','Zero')
axis([-2 0 -2 2])
grid on;
set(gca, 'LooseInset', get(gca, 'TightInset'));

if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(2)
    garyfyFigure
    myprint(2,'RocketPDCont_NomInput')
    
    figure(3)
    garyfyFigure
    myprint(3,'RocketPDCont_NomOutput')
    
    figure(4)
    garyfyFigure
    myprint(4,'RocketPDCont_NomError')
    
    figure(5)
    garyfyFigure
    myprint(5,'RocketPDCont_NomPoles')
end
